#include "highlighter.h"

Highlighter::Highlighter(QTextDocument *parent)
    : QSyntaxHighlighter(parent)
{
}

bool Highlighter::setSyntaxSettings(QString extention)
{
    QDomDocument doc(":/settings/text_editor.xml");
    QFile file(":/settings/text_editor.xml");
    if (!file.open(QIODevice::ReadOnly)) {
        qDebug() << "Open file error";
        return 0;
    }

    if (!doc.setContent(&file)) {
        file.close();
        return 0;
    }
    file.close();

    highlightingRules.clear();

    QDomNode syntaxes = doc.documentElement().namedItem("syntaxes");
    QDomElement pattern, format;
    HighlightingRule hrule;
    QString nodename;

    for(auto syntax = syntaxes.firstChild(); !syntax.isNull(); syntax = syntax.nextSibling()) {
        if( !syntax.toElement().attribute("ext_list").contains(extention) ) continue;

        for(auto rule = syntax.firstChild(); !rule.isNull(); rule = rule.nextSibling()) {
            nodename = rule.toElement().tagName();
            if(nodename == "rule") {
                pattern = rule.namedItem("pattern").toElement();
                format = rule.namedItem("format").toElement();

                hrule.pattern = QRegularExpression(pattern.attribute("value"));
                hrule.format.setForeground(QColor(format.attribute("foreground")));
                hrule.format.setFontWeight(format.attribute("font_weight").toInt());

                highlightingRules.append(hrule);
            } else

            if(nodename == "startComment") {
                pattern = rule.namedItem("pattern").toElement();
                commentStartExpression = QRegularExpression(pattern.attribute("value"));
            }else

            if(nodename == "endComment") {
                pattern = rule.namedItem("pattern").toElement();
                commentEndExpression = QRegularExpression(pattern.attribute("value"));
            }else

            if(nodename == "multiLineComment") {
                format = rule.namedItem("format").toElement();
                multiLineCommentFormat.setForeground(QColor(format.attribute("foreground")));
            }
        }
    }
    return 1;
}


void Highlighter::highlightBlock(const QString &text)
{
    if(highlightingRules.isEmpty()) return;

    for (const HighlightingRule &rule : qAsConst(highlightingRules)) {
        QRegularExpressionMatchIterator matchIterator = rule.pattern.globalMatch(text);
        while (matchIterator.hasNext()) {
            QRegularExpressionMatch match = matchIterator.next();
            setFormat(match.capturedStart(), match.capturedLength(), rule.format);
        }
    }

    setCurrentBlockState(0);

    int startIndex = 0;
    if (previousBlockState() != 1)
        startIndex = text.indexOf(commentStartExpression);
    while (startIndex >= 0) {
        QRegularExpressionMatch match = commentEndExpression.match(text, startIndex);
        int endIndex = match.capturedStart();
        int commentLength = 0;
        if (endIndex == -1) {
          setCurrentBlockState(1);
          commentLength = text.length() - startIndex;
        } else {
          commentLength = endIndex - startIndex + match.capturedLength();
        }
        setFormat(startIndex, commentLength, multiLineCommentFormat);
        startIndex = text.indexOf(commentStartExpression, startIndex + commentLength);
    }
}
